@@ -1,12 +1,13 @@
 /*
- *  $Id: editbox.c,v 1.63 2015/01/25 22:57:49 tom Exp $
+ *  $Id: editbox.c,v 1.73 2019/07/24 23:32:46 tom Exp $
  *
  *  editbox.c -- implements the edit box
  *
- *  Copyright 2007-2013,2015 Thomas E. Dickey
+ *  Copyright 2007-2018,2019 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,11 +55,8 @@ grow_list(char ***list, int *have, int want)
 static void
 load_list(const char *file, char ***list, int *rows)
 {
-    FILE *fp;
     char *blob = 0;
     struct stat sb;
-    unsigned n, pass;
-    unsigned need;
     size_t size;
 
     *list = 0;
@@ -69,9 +67,12 @@ load_list(const char *file, char ***list, int *rows)
 	dlg_exiterr("Not a file: %s", file);
 
     size = (size_t) sb.st_size;
-    if ((blob = dlg_malloc(char, size + 1)) == 0) {
+    if ((blob = dlg_malloc(char, size + 2)) == 0) {
 	fail_list();
     } else {
+	FILE *fp;
+	unsigned n, pass;
+
 	blob[size] = '\0';
 
 	if ((fp = fopen(file, "r")) == 0)
@@ -79,9 +80,18 @@ load_list(const char *file, char ***list, int *rows)
 	size = fread(blob, sizeof(char), size, fp);
 	fclose(fp);
 
+	/*
+	 * If the file is not empty, ensure that it ends with a newline.
+	 */
+	if (size != 0 && blob[size - 1] != '\n') {
+	    blob[++size - 1] = '\n';
+	    blob[size] = '\0';
+	}
+
 	for (pass = 0; pass < 2; ++pass) {
 	    int first = TRUE;
-	    need = 0;
+	    unsigned need = 0;
+
 	    for (n = 0; n < size; ++n) {
 		if (first && pass) {
 		    (*list)[need] = blob + n;
@@ -277,9 +287,10 @@ static int
 widest_line(char **list)
 {
     int result = MAX_LEN;
-    char *value;
 
     if (list != 0) {
+	char *value;
+
 	while ((value = *list++) != 0) {
 	    int check = (int) strlen(value);
 	    if (check > result)
@@ -318,6 +329,7 @@ dlg_editbox(const char *title,
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
 	NAVIGATE_BINDINGS,
+	TOGGLEKEY_BINDINGS,
 	END_KEYS_BINDING
     };
     static DLG_KEYS_BINDING binding2[] = {
@@ -325,6 +337,7 @@ dlg_editbox(const char *title,
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
 	NAVIGATE_BINDINGS,
+	/* no TOGGLEKEY_BINDINGS, since that includes space... */
 	END_KEYS_BINDING
     };
     /* *INDENT-ON* */
@@ -354,6 +367,12 @@ dlg_editbox(const char *title,
     const char **buttons = dlg_ok_labels();
     int mincols = (3 * COLS / 4);
 
+    DLG_TRACE(("# editbox args:\n"));
+    DLG_TRACE2S("title", title);
+    /* FIXME dump the rows & list */
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+
     dlg_save_vars(&save_vars);
     dialog_vars.separate_output = TRUE;
 
@@ -389,7 +408,7 @@ dlg_editbox(const char *title,
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
 
     /* Draw the editing field in a box */
     box_y = MARGIN + 0;
@@ -517,13 +536,18 @@ dlg_editbox(const char *title,
 	    && (key >= KEY_MAX)) {
 	    int wide = getmaxx(editing);
 	    int cell = key - KEY_MAX;
-	    thisrow = (cell / wide) + base_row;
-	    col_offset = (cell % wide);
-	    chr_offset = col_to_chr_offset(THIS_ROW, col_offset);
-	    show_one = TRUE;
-	    if (state != sTEXT) {
-		state = sTEXT;
-		show_buttons = TRUE;
+	    int check = (cell / wide) + base_row;
+	    if (check < listsize) {
+		thisrow = check;
+		col_offset = (cell % wide);
+		chr_offset = col_to_chr_offset(THIS_ROW, col_offset);
+		show_one = TRUE;
+		if (state != sTEXT) {
+		    state = sTEXT;
+		    show_buttons = TRUE;
+		}
+	    } else {
+		beep();
 	    }
 	    continue;
 	} else if (was_mouse && key >= KEY_MIN) {
@@ -674,27 +698,31 @@ dlg_editbox(const char *title,
 		break;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
-		/* repaint */
 		dlg_clear();
+		dlg_unregister_window(editing);
 		dlg_del_window(editing);
 		dlg_del_window(dialog);
-		refresh();
 		dlg_mouse_free_regions();
+		/* repaint */
 		goto retry;
 #endif
+	    case DLGK_TOGGLE:
+		if (state != sTEXT) {
+		    result = dlg_ok_buttoncode(state);
+		} else {
+		    beep();
+		}
+		break;
 	    default:
 		beep();
 		break;
 	    }
-	} else {
-	    if ((key == ' ') && (state != sTEXT)) {
-		result = dlg_ok_buttoncode(state);
-	    } else {
-		beep();
-	    }
+	} else if (key > 0) {
+	    beep();
 	}
     }
 
