@@ -1,9 +1,9 @@
 /*
- *  $Id: fselect.c,v 1.96 2017/01/31 00:26:04 tom Exp $
+ *  $Id: fselect.c,v 1.104 2019/07/24 23:40:15 tom Exp $
  *
  *  fselect.c -- implements the file-selector box
  *
- *  Copyright 2000-2016,2017	Thomas E. Dickey
+ *  Copyright 2000-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -125,9 +125,9 @@ data_of(LIST * list)
 static void
 free_list(LIST * list, int reinit)
 {
-    int n;
-
     if (list->data != 0) {
+	int n;
+
 	for (n = 0; list->data[n] != 0; n++)
 	    free(list->data[n]);
 	free(list->data);
@@ -173,13 +173,14 @@ keep_visible(LIST * list)
 static int
 find_choice(char *target, LIST * list)
 {
-    int n;
     int choice = list->choice;
-    int len_1, len_2, cmp_1, cmp_2;
 
     if (*target == 0) {
 	list->choice = 0;
     } else {
+	int n;
+	int len_1, cmp_1;
+
 	/* find the match with the longest length.  If more than one has the
 	 * same length, choose the one with the closest match of the final
 	 * character.
@@ -189,6 +190,7 @@ find_choice(char *target, LIST * list)
 	for (n = 0; n < list->length; n++) {
 	    char *a = target;
 	    char *b = list->data[n];
+	    int len_2, cmp_2;
 
 	    len_2 = 0;
 	    while ((*a != 0) && (*b != 0) && (*a == *b)) {
@@ -216,13 +218,13 @@ find_choice(char *target, LIST * list)
 static void
 display_list(LIST * list)
 {
-    int n;
-    int x;
-    int y;
-    int top;
-    int bottom;
-
     if (list->win != 0) {
+	int n;
+	int x;
+	int y;
+	int top;
+	int bottom;
+
 	dlg_attr_clear(list->win, getmaxy(list->win), getmaxx(list->win), item_attr);
 	for (n = list->offset; n < list->length && list->data[n]; n++) {
 	    y = n - list->offset;
@@ -230,11 +232,11 @@ display_list(LIST * list)
 		break;
 	    (void) wmove(list->win, y, 0);
 	    if (n == list->choice)
-		(void) wattrset(list->win, item_selected_attr);
+		dlg_attrset(list->win, item_selected_attr);
 	    (void) waddstr(list->win, list->data[n]);
-	    (void) wattrset(list->win, item_attr);
+	    dlg_attrset(list->win, item_attr);
 	}
-	(void) wattrset(list->win, item_attr);
+	dlg_attrset(list->win, item_attr);
 
 	getparyx(list->win, y, x);
 
@@ -264,16 +266,17 @@ display_list(LIST * list)
  * that is really required is that they're distinct, so we can put them in a
  * switch statement.
  */
+#if USE_MOUSE
 static void
 fix_arrows(LIST * list)
 {
-    int x;
-    int y;
-    int top;
-    int right;
-    int bottom;
-
     if (list->win != 0) {
+	int x;
+	int y;
+	int top;
+	int right;
+	int bottom;
+
 	getparyx(list->win, y, x);
 	top = y - 1;
 	right = getmaxx(list->win);
@@ -289,11 +292,14 @@ fix_arrows(LIST * list)
 			: KEY_NPAGE));
     }
 }
+#else
+#define fix_arrows(list)	/* nothing */
+#endif
 
-static int
-show_list(char *target, LIST * list, int keep)
+static bool
+show_list(char *target, LIST * list, bool keep)
 {
-    int changed = keep || find_choice(target, list);
+    bool changed = keep || find_choice(target, list);
     display_list(list);
     return changed;
 }
@@ -302,12 +308,12 @@ show_list(char *target, LIST * list, int keep)
  * Highlight the closest match to 'target' in the given list, setting offset
  * to match.
  */
-static int
-show_both_lists(char *input, LIST * d_list, LIST * f_list, int keep)
+static bool
+show_both_lists(char *input, LIST * d_list, LIST * f_list, bool keep)
 {
     char *leaf = leaf_of(input);
 
-    return show_list(leaf, d_list, keep) | show_list(leaf, f_list, keep);
+    return show_list(leaf, d_list, keep) || show_list(leaf, f_list, keep);
 }
 
 /*
@@ -386,7 +392,6 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
     char *test;
     size_t test_len;
     size_t i;
-    int j;
     char *buff;
 
     match(name, d_list, f_list, &match_list);
@@ -406,6 +411,8 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
 	    i++;
 	}
     } else {
+	int j;
+
 	for (i = 0; i < test_len; i++) {
 	    char test_char = test[i];
 	    if (test_char == '\0')
@@ -429,16 +436,13 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
 }
 
 static bool
-fill_lists(char *current, char *input, LIST * d_list, LIST * f_list, int keep)
+fill_lists(char *current, char *input, LIST * d_list, LIST * f_list, bool keep)
 {
     bool result = TRUE;
     bool rescan = FALSE;
-    DIR *dp;
-    DIRENT *de;
     struct stat sb;
     int n;
     char path[MAX_LEN + 1];
-    char *leaf;
 
     /* check if we've updated the lists */
     for (n = 0; current[n] && input[n]; n++) {
@@ -457,7 +461,9 @@ fill_lists(char *current, char *input, LIST * d_list, LIST * f_list, int keep)
     }
 
     if (rescan) {
+	DIR *dp;
 	size_t have = strlen(input);
+	char *leaf;
 
 	if (have > MAX_LEN)
 	    have = MAX_LEN;
@@ -477,8 +483,14 @@ fill_lists(char *current, char *input, LIST * d_list, LIST * f_list, int keep)
 	}
 	DLG_TRACE(("opendir '%s'\n", path));
 	if ((dp = opendir(path)) != 0) {
+	    DIRENT *de;
+
 	    while ((de = readdir(dp)) != 0) {
-		strncpy(leaf, de->d_name, NAMLEN(de))[NAMLEN(de)] = 0;
+		size_t len = NAMLEN(de);
+		if (len == 0 || (len + have + 2) >= MAX_LEN)
+		    continue;
+		memcpy(leaf, de->d_name, len);
+		leaf[len] = '\0';
 		if (stat(path, &sb) == 0) {
 		    if ((sb.st_mode & S_IFMT) == S_IFDIR)
 			add_to_list(d_list, leaf);
@@ -586,8 +598,8 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
     int result = DLG_EXIT_UNKNOWN;
     int state = dialog_vars.default_button >= 0 ? dlg_default_button() : sTEXT;
     int button;
-    int first = (state == sTEXT);
-    int first_trace = TRUE;
+    bool first = (state == sTEXT);
+    bool first_trace = TRUE;
     char *input;
     char *completed;
     char current[MAX_LEN + 1];
@@ -602,6 +614,12 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
     int min_items = height ? 0 : 4;
     LIST d_list, f_list;
 
+    DLG_TRACE(("# %s args:\n", dselect ? "dselect" : "fselect"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("path", path);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+
     dlg_does_output();
 
     /* Set up the initial value */
@@ -633,7 +651,7 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
 
     /* Draw the input field box */
     tbox_height = 1;
@@ -715,7 +733,7 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 	if (resized) {
 	    resized = FALSE;
 	    dlg_show_string(w_text, input, offset, inputbox_attr,
-			    0, 0, tbox_width, (bool) 0, (bool) first);
+			    0, 0, tbox_width, FALSE, first);
 	}
 #endif
 
@@ -843,6 +861,7 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 		continue;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
