@@ -1,9 +1,9 @@
 /*
- *  $Id: menubox.c,v 1.145 2012/12/30 21:11:02 tom Exp $
+ *  $Id: menubox.c,v 1.161 2019/07/24 23:42:20 tom Exp $
  *
  *  menubox.c -- implements the menu box
  *
- *  Copyright 2000-2011,2012	Thomas E. Dickey
+ *  Copyright 2000-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public Licens, version 2.1e
@@ -24,7 +24,7 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 typedef enum {
@@ -68,7 +68,6 @@ print_item(ALL_DATA * data,
 	   bool is_inputmenu)
 {
     chtype save = dlg_get_attrs(win);
-    int n;
     int climit = (data->item_x - data->tag_x - GUTTER);
     int my_width = data->menu_width;
     int my_x = data->item_x;
@@ -95,13 +94,15 @@ print_item(ALL_DATA * data,
 
     /* Clear 'residue' of last item and mark current current item */
     if (is_inputmenu) {
-	(void) wattrset(win, (selected != Unselected) ? item_selected_attr : item_attr);
+	int n;
+
+	dlg_attrset(win, (selected != Unselected) ? item_selected_attr : item_attr);
 	for (n = my_y - 1; n < my_y + INPUT_ROWS - 1; n++) {
 	    wmove(win, n, 0);
 	    wprintw(win, "%*s", my_width, " ");
 	}
     } else {
-	(void) wattrset(win, menubox_attr);
+	dlg_attrset(win, menubox_attr);
 	wmove(win, my_y, 0);
 	wprintw(win, "%*s", my_width, " ");
     }
@@ -131,7 +132,7 @@ print_item(ALL_DATA * data,
     if (selected) {
 	dlg_item_help(item->help);
     }
-    (void) wattrset(win, save);
+    dlg_attrset(win, save);
 }
 
 /*
@@ -147,7 +148,7 @@ input_menu_edit(ALL_DATA * data,
     char *result;
     int offset = 0;
     int key = 0, fkey = 0;
-    int first = TRUE;
+    bool first = TRUE;
     /* see above */
     bool is_inputmenu = TRUE;
     int y = ItemToRow(choice);
@@ -182,7 +183,7 @@ input_menu_edit(ALL_DATA * data,
 	}
     }
     print_item(data, data->menu, items, choice, Selected, TRUE);
-    (void) wattrset(data->menu, save);
+    dlg_attrset(data->menu, save);
 
     *resultp = result;
     return code;
@@ -191,19 +192,16 @@ input_menu_edit(ALL_DATA * data,
 static int
 handle_button(int code, DIALOG_LISTITEM * items, int choice)
 {
+    char *help_result;
+
     switch (code) {
     case DLG_EXIT_OK:		/* FALLTHRU */
     case DLG_EXIT_EXTRA:
 	dlg_add_string(items[choice].name);
 	break;
     case DLG_EXIT_HELP:
-	dlg_add_result("HELP ");
-	if (USE_ITEM_HELP(items[choice].help)) {
-	    dlg_add_string(items[choice].help);
-	    code = DLG_EXIT_ITEM_HELP;
-	} else {
-	    dlg_add_string(items[choice].name);
-	}
+	dlg_add_help_listitem(&code, &help_result, &items[choice]);
+	dlg_add_string(help_result);
 	break;
     }
     return code;
@@ -248,7 +246,7 @@ print_menu(ALL_DATA * data, int choice, int scrollamt, int max_choice, bool is_i
     if (is_inputmenu) {
 	int spare_lines, x_count;
 	spare_lines = data->menu_height % INPUT_ROWS;
-	(void) wattrset(data->menu, menubox_attr);
+	dlg_attrset(data->menu, menubox_attr);
 	for (; spare_lines; spare_lines--) {
 	    wmove(data->menu, data->menu_height - spare_lines, 0);
 	    for (x_count = 0; x_count < data->menu_width;
@@ -307,7 +305,7 @@ dlg_menu(const char *title,
     static DLG_KEYS_BINDING binding[] = {
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
-	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	' ' ),
+	TOGGLEKEY_BINDINGS,
 	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_RIGHT ),
 	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	TAB ),
 	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_BTAB ),
@@ -334,6 +332,8 @@ dlg_menu(const char *title,
     /* *INDENT-ON* */
 
 #ifdef KEY_RESIZE
+    int old_LINES = LINES;
+    int old_COLS = COLS;
     int old_height = height;
     int old_width = width;
 #endif
@@ -348,21 +348,36 @@ dlg_menu(const char *title,
     int found;
     int use_width, name_width, text_width, list_width;
     WINDOW *dialog, *menu;
-    char *prompt = dlg_strclone(cprompt);
+    char *prompt = 0;
     const char **buttons = dlg_ok_labels();
     bool is_inputmenu = ((rename_menutext != 0)
 			 && (rename_menutext != dlg_dummy_menutext));
 
+    DLG_TRACE(("# menubox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("lheight", menu_height);
+    DLG_TRACE2N("llength", item_no);
+    /* FIXME dump the items[][] too */
+    DLG_TRACE2N("current", *current_item);
+    DLG_TRACE2N("rename", rename_menutext != 0);
+
+    dialog_state.plain_buttons = TRUE;
+
     all.items = items;
     all.item_no = item_no;
 
     dlg_does_output();
-    dlg_tab_correct_str(prompt);
 
 #ifdef KEY_RESIZE
   retry:
 #endif
 
+    prompt = dlg_strclone(cprompt);
+    dlg_tab_correct_str(prompt);
+
     all.menu_height = menu_height;
     use_width = dlg_calc_list_width(item_no, items) + 10;
     use_width = MAX(26, use_width);
@@ -394,7 +409,7 @@ dlg_menu(const char *title,
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
     dlg_print_autowrap(dialog, prompt, height, width);
 
     all.menu_width = width - 6;
@@ -645,6 +660,7 @@ dlg_menu(const char *title,
 		dlg_draw_buttons(dialog, height - 2, 0, buttons, button,
 				 FALSE, width);
 		break;
+	    case DLGK_TOGGLE:
 	    case DLGK_ENTER:
 		if (is_inputmenu)
 		    result = dlg_ok_buttoncode(button);
@@ -706,14 +722,15 @@ dlg_menu(const char *title,
 		break;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
-		height = old_height;
-		width = old_width;
-		/* repaint */
+		resizeit(height, LINES);
+		resizeit(width, COLS);
+		free(prompt);
 		dlg_clear();
 		dlg_del_window(dialog);
-		refresh();
 		dlg_mouse_free_regions();
+		/* repaint */
 		goto retry;
 #endif
 	    default:
