@@ -1,9 +1,9 @@
 /*
- *  $Id: ui_getc.c,v 1.66 2012/11/30 01:56:14 tom Exp $
+ *  $Id: ui_getc.c,v 1.71 2019/07/25 00:07:15 tom Exp $
  *
  *  ui_getc.c - user interface glue for getc()
  *
- *  Copyright 2001-2011,2012	Thomas E. Dickey
+ *  Copyright 2001-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -89,10 +89,16 @@ dlg_remove_callback(DIALOG_CALLBACK * p)
     DIALOG_CALLBACK *q;
 
     if (p->input != 0) {
-	fclose(p->input);
+	FILE *input = p->input;
+	fclose(input);
 	if (p->input == dialog_state.pipe_input)
 	    dialog_state.pipe_input = 0;
-	p->input = 0;
+	/* more than one callback can have the same input */
+	for (q = dialog_state.getc_callbacks; q != 0; q = q->next) {
+	    if (q->input == input) {
+		q->input = 0;
+	    }
+	}
     }
 
     if (!(p->keep_win))
@@ -147,8 +153,9 @@ handle_inputs(WINDOW *win)
     if (result) {
 	(void) wmove(win, cur_y, cur_x);	/* Restore cursor position */
 	wrefresh(win);
-	curs_set(state);
     }
+    if (state != ERR)
+	curs_set(state);
     return result;
 }
 
@@ -180,15 +187,17 @@ check_inputs(void)
     DIALOG_CALLBACK *p;
     fd_set read_fds;
     struct timeval test;
-    int last_fd = -1;
-    int fd;
-    int found;
     int result = -1;
 
     if ((p = dialog_state.getc_callbacks) != 0) {
+	int last_fd = -1;
+	int found;
+	int fd;
+
 	FD_ZERO(&read_fds);
 
 	while (p != 0) {
+
 	    p->input_ready = FALSE;
 	    if (p->input != 0 && (fd = fileno(p->input)) >= 0) {
 		FD_SET(fd, &read_fds);
@@ -283,6 +292,36 @@ dlg_flush_getc(void)
 #endif
 }
 
+/*
+ * Report the last key entered by the user.  The 'mode' parameter controls
+ * the way it is separated from other results:
+ * -2 (no separator)
+ * -1 (separator after the key name)
+ * 0 (separator is optionally before the key name)
+ * 1 (same as -1)
+ */
+void
+dlg_add_last_key(int mode)
+{
+    if (dialog_vars.last_key) {
+	if (mode >= 0) {
+	    if (mode > 0) {
+		dlg_add_last_key(-1);
+	    } else {
+		if (dlg_need_separator())
+		    dlg_add_separator();
+		dlg_add_last_key(-2);
+	    }
+	} else {
+	    char temp[80];
+	    sprintf(temp, "%d", last_getc);
+	    dlg_add_string(temp);
+	    if (mode == -1)
+		dlg_add_separator();
+	}
+    }
+}
+
 /*
  * Check if the stream has been unexpectedly closed, returning false in that
  * case.
@@ -306,9 +345,7 @@ really_getch(WINDOW *win, int *fkey)
 {
     int ch;
 #ifdef USE_WIDE_CURSES
-    int code;
     mbstate_t state;
-    wchar_t my_wchar;
     wint_t my_wint;
 
     /*
@@ -316,6 +353,9 @@ really_getch(WINDOW *win, int *fkey)
      * having to change the rest of the code to use wide-characters.
      */
     if (used_last_getc >= have_last_getc) {
+	int code;
+	wchar_t my_wchar;
+
 	used_last_getc = 0;
 	have_last_getc = 0;
 	ch = ERR;
@@ -450,10 +490,12 @@ dlg_getc(WINDOW *win, int *fkey)
 	    case ERR:		/* wtimeout() in effect; check for file I/O */
 		if (interval > 0
 		    && current >= expired) {
-		    dlg_exiterr("timeout");
-		}
-		if (!valid_file(stdin)
-		    || !valid_file(dialog_state.screen_output)) {
+		    DLG_TRACE(("# dlg_getc: timeout expired\n"));
+		    ch = ESC;
+		    done = TRUE;
+		} else if (!valid_file(stdin)
+			   || !valid_file(dialog_state.screen_output)) {
+		    DLG_TRACE(("# dlg_getc: input or output is invalid\n"));
 		    ch = ESC;
 		    done = TRUE;
 		} else if (check_inputs()) {
@@ -557,7 +599,6 @@ void
 dlg_killall_bg(int *retval)
 {
     DIALOG_CALLBACK *cb;
-    int pid;
 #ifdef HAVE_TYPE_UNIONWAIT
     union wait wstatus;
 #else
@@ -574,6 +615,7 @@ dlg_killall_bg(int *retval)
 	    }
 	}
 	if (dialog_state.getc_callbacks != 0) {
+	    int pid;
 
 	    refresh();
 	    fflush(stdout);
